using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib2
{
    public class UWB_XNACircleGeometry
    {
        protected Vector3 m_center;
        protected float m_radius;

        public UWB_XNACircleGeometry()
        {
            m_center = new Vector3(0, 0, 0);
            m_radius = 0;
        }

        public void SetCenter(float x, float y)
        {
            m_center.X = x;
            m_center.Y = y;
        }

        public void SetCenter(Vector3 center)
        {
            m_center = center;
        }

        public Vector3 GetCenter()
        {
            return m_center;
        }

        public void SetRadius(float radius)
        {
            m_radius = radius;
        }

        public float GetRadius()
        {
            return m_radius;
        }

        public void Draw()
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on Circle2D draw call");
            if (m_radius > 0)
            {
                const int kNumPts = 100;
                DeviceVertexFormat[] v = new DeviceVertexFormat[kNumPts + 2];

                float theta = (float)((2.0f * 3.14159265f) / kNumPts);

                v[0].Position = new Vector3(m_center.X, m_center.Y, 0f);
                v[0].Color = new Color(new Vector4(.4f, .4f, .4f, 1f));

                for (int i = 0; i < (kNumPts); i++)
                {
                    float X = m_center.X + (m_radius * (float)Math.Cos(i * theta));
                    float Y = m_center.Y + (m_radius * (float)Math.Sin(i * theta));
                    v[i + 1].Position = new Vector3(X, Y, 0f);
                    v[i + 1].Color = v[0].Color;
                }
                v[kNumPts + 1] = v[1];

                device.DrawUserPrimitives(PrimitiveType.TriangleFan, v, 0, kNumPts);
            }
        }
    }
}
